# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

from django.contrib.auth.middleware import AuthenticationMiddleware
from django.contrib.auth.models import User
from django.contrib.sessions.middleware import SessionMiddleware
from django.test.client import RequestFactory
from django.test.testcases import TestCase

from m3_auth.backends.base.remote.protocol import BaseProtocol
from m3_auth.backends.base.remote.user import BaseRemoteUser
from m3_auth.backends.oauth2 import OAuthBackend
from m3_auth.backends.oauth2.exceptions import RefreshTokenError
from m3_auth.utils import is_authenticated
from m3_auth.config import Config


class TestOAuthProtocol(BaseProtocol):

    """Тестовый протокол, для проверки OAuthBackend'а."""

    def refresh_token(self, request):
        if 'raise' in request.POST:
            raise RefreshTokenError()
        return True

    def get_logout_url(self, request):
        pass

    def process_logout(self, request):
        pass

    def process_login(self, request):
        pass

    def get_login_url(self, request):
        pass

    @classmethod
    def from_config(cls, config):
        pass


class TestOAuthUser(BaseRemoteUser):

    """Тестовый OAuth пользователь."""


class TestOAuthBackend(TestCase):

    """Проверка OAuth бэкенда."""

    def setUp(self):
        super(TestOAuthBackend, self).setUp()
        self.factory = RequestFactory()
        self.user = User.objects.get(username='admin')

        protocol = TestOAuthProtocol(TestOAuthUser, Config())
        self.backend = OAuthBackend('oauth', protocol, [])

    def test_refresh_token(self):
        """Проверка метода refresh_token."""
        request = self.factory.post('/')
        # Все ок, ничего не должно произойти
        self.assertIsNone(self.backend.refresh_token(request))

        request = self.factory.post('/', {'raise': True})
        self.apply_middleware(request)
        request.user = self.user

        # Должно быть перехвачено исключение, и выкинуть пользователя
        self.backend.refresh_token(request)
        self.assertFalse(is_authenticated(request))

    @staticmethod
    def apply_middleware(request):
        SessionMiddleware().process_request(request)
        AuthenticationMiddleware().process_request(request)
